// Earth model

// New version 2022: Uses DrawModel to simplify the code.
// The whole model loading is not what this is about.

#define MAIN
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "LittleOBJLoader.h"
#include "LoadTGA.h"
// uses framework Cocoa
// uses framework OpenGL

mat4 translationMatrix;
mat4 projectionMatrix;

// vertex array object
unsigned int bunnyVertexArrayObjID;
Model *m;
// Reference to shader program
GLuint program;
GLuint tex, gloss;

GLfloat speed = 0.1;
GLint texon = 0;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_TRUE);
	printError("GL inits");

	// Load and compile shader
	program = loadShaders("tex.vert", "tex.frag");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	m = LoadModel("sphere.obj");
	
	translationMatrix = T(0,0,-3);
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 30.0);
	
	glUniformMatrix4fv(glGetUniformLocation(program, "camMatrix"), 1, GL_TRUE, translationMatrix.m); // A "look-at" function could be more practical
	glUniformMatrix4fv(glGetUniformLocation(program, "projMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	printError("init arrays");
	
	glUniform1i(glGetUniformLocation(program, "tex"), 0); // Texture unit 0
	glUniform1i(glGetUniformLocation(program, "gloss"), 1); // Texture unit 1
	glActiveTexture(GL_TEXTURE0); // Just make sure the texture unit match
	LoadTGATextureSimple("Earth.tga", &tex);
	glActiveTexture(GL_TEXTURE1); // Just make sure the texture unit match
	LoadTGATextureSimple("Earthgloss.tga", &gloss);
	glUniform1i(glGetUniformLocation(program, "texon"), texon);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
}

GLfloat a = 0.0;

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	a += speed; // 0.1;
	mat4 total;

	total = Ry(a/2) * T(0, 0, 0);
	glUniformMatrix4fv(glGetUniformLocation(program, "mdlMatrix"), 1, GL_TRUE, total.m);

	DrawModel(m, program, "inPosition", "inNormal", "inTexCoord");
	
	printError("display");
	
	glutSwapBuffers();
}

void keydown(unsigned char key, int x, int y)
{
	if (key == '+')
		speed += 0.01;
	if (key == '-')
		speed -= 0.01;
	if (key == ' ')
		texon = 1 - texon;
	
	glUniform1i(glGetUniformLocation(program, "texon"), texon);
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow ("Gloss mapping example");
	glutDisplayFunc(display);
	glutKeyboardFunc(keydown);
	init ();
	glutRepeatingTimer(20);
	glutMainLoop();
	exit(0);
}
